@@ -38,7 +38,8 @@
 (defcustom riece-saved-forms
   '(riece-server-alist
     riece-channel-buffer-mode
-    riece-user-list-buffer-mode)
+    riece-user-list-buffer-mode
+    riece-layout)
   "Variables saved after each session is completed."
   :type 'string
   :group 'riece-options)
@@ -107,7 +108,8 @@ way is to put Riece variables on .emacs or file loaded from there."
 			  riece-history
 			  riece-url
 			  riece-button
-			  riece-menu)
+			  riece-menu
+			  riece-icon)
   "Add-ons insinuated into Riece."
   :type '(repeat symbol)
   :group 'riece-options)
@@ -197,7 +199,8 @@ way is to put Riece variables on .emacs or file loaded from there."
 
 (defcustom riece-quit-timeout 10
   "Quit timeout when there is no response from server."
-  :type 'integer
+  :type '(radio (integer :tag "Seconds")
+		(const nil))
   :group 'riece-server)
 
 (defcustom riece-channel-buffer-mode t
@@ -241,7 +244,7 @@ way is to put Riece variables on .emacs or file loaded from there."
   :type '(radio (string :tag "Away message"))
   :group 'riece-options)
 
-(defcustom riece-gather-channel-modes t
+(defcustom riece-gather-channel-modes nil
   "If non-nil, gather channel modes when we join a channel."
   :type 'boolean
   :group 'riece-options)
@@ -251,6 +254,17 @@ way is to put Riece variables on .emacs or file loaded from there."
   :type 'function
   :group 'riece-options)
 
+(defcustom riece-shrink-buffer-idle-time-delay 5
+  "Number of idle seconds to wait before shrinking channel buffers."
+  :type 'integer
+  :group 'riece-options)
+
+(defcustom riece-max-buffer-size 65535
+  "Maximum size of channel buffers."
+  :type '(radio (integer :tag "Number of characters")
+		(const nil))
+  :group 'riece-options)
+
 (defcustom riece-format-time-function #'current-time-string
   "Function to convert the specified time to the human readable form."
   :type 'function
