@@ -30,6 +30,8 @@
 (require 'riece-compat)
 (require 'riece-commands)
 
+(autoload 'derived-mode-class "derived")
+
 (defvar riece-channel-list-mode-map (make-sparse-keymap))
 (defvar riece-user-list-mode-map (make-sparse-keymap))
 
@@ -57,6 +59,9 @@
     (riece-channel-list-buffer "*Channels*" riece-channel-list-mode)
     (riece-user-list-buffer " *Users*" riece-user-list-mode)))
 
+(defvar riece-shrink-buffer-idle-timer nil
+  "Timer object to periodically shrink channel buffers.")
+
 (defvar riece-select-keys
   `("1" riece-command-switch-to-channel-by-number-1
     "2" riece-command-switch-to-channel-by-number-2
@@ -201,10 +206,10 @@ If optional argument SAFE is nil, overwrite previous definitions."
     "o" riece-command-set-operators
     "v" riece-command-set-voices
     "f" riece-command-finger
-    " " riece-command-nick-scroll-up
-    "\177" riece-command-nick-scroll-down
-    [delete] riece-command-nick-scroll-down
-    [backspace] riece-command-nick-scroll-down
+    " " riece-command-user-list-scroll-up
+    "\177" riece-command-user-list-scroll-down
+    [delete] riece-command-user-list-scroll-down
+    [backspace] riece-command-user-list-scroll-down
     "c" riece-command-select-command-buffer)
 
   (riece-define-keys riece-channel-list-mode-map
@@ -259,17 +264,46 @@ If optional argument CONFIRM is non-nil, ask which IRC server to connect."
   (riece-insinuate-addons riece-addons)
   (run-hooks 'riece-after-load-startup-hook)
   (if (riece-server-opened)
-      (error "Already running"))
-  (if (or confirm (null riece-server))
-      (setq riece-server (completing-read "Server: " riece-server-alist)))
-  (if (stringp riece-server)
-      (setq riece-server (riece-server-name-to-server riece-server)))
-  (riece-create-buffers)
-  (switch-to-buffer riece-command-buffer)
-  (riece-redisplay-buffers)
-  (riece-open-server riece-server "")
-  (run-hooks 'riece-startup-hook)
-  (message "%s" (substitute-command-keys "Type \\[describe-mode] for help")))
+      (riece-command-configure-windows)
+    (if (or confirm (null riece-server))
+	(setq riece-server (completing-read "Server: " riece-server-alist)))
+    (if (stringp riece-server)
+	(setq riece-server (riece-server-name-to-server riece-server)))
+    (riece-create-buffers)
+    (if riece-max-buffer-size
+	(setq riece-shrink-buffer-idle-timer
+	      (riece-run-with-idle-timer
+	       riece-shrink-buffer-idle-time-delay t
+	       (lambda ()
+		 (let ((buffers riece-buffer-list))
+		   (while buffers
+		     (if (eq (derived-mode-class
+			      (with-current-buffer (car buffers)
+				major-mode))
+			     'riece-dialogue-mode)
+			 (riece-shrink-buffer (car buffers)))
+		     (setq buffers (cdr buffers))))))))
+    (switch-to-buffer riece-command-buffer)
+    (riece-redisplay-buffers)
+    (riece-open-server riece-server "")
+    (run-hooks 'riece-startup-hook)
+    (message "%s" (substitute-command-keys
+		   "Type \\[describe-mode] for help"))))
+
+(defun riece-shrink-buffer (buffer)
+  (save-excursion
+    (set-buffer buffer)
+    (goto-char (point-min))
+    (while (> (buffer-size) riece-max-buffer-size)
+      (let* ((inhibit-read-only t)
+	     buffer-read-only
+	     (start (point))
+	     (end (progn (beginning-of-line 2) (point)))
+	     (overlays (riece-overlays-in start end)))
+	(while overlays
+	  (riece-delete-overlay (car overlays))
+	  (setq overlays (cdr overlays)))
+	(delete-region start end)))))
 
 (defun riece-exit ()
   (if riece-save-variables-are-dirty
@@ -279,6 +313,8 @@ If optional argument CONFIRM is non-nil, ask which IRC server to connect."
 	     (buffer-live-p (car riece-buffer-list)))
 	(funcall riece-buffer-dispose-function (car riece-buffer-list)))
     (setq riece-buffer-list (cdr riece-buffer-list)))
+  (if riece-shrink-buffer-idle-timer
+      (riece-cancel-timer riece-shrink-buffer-idle-timer))
   (setq riece-server nil
 	riece-current-channels nil
 	riece-current-channel nil
@@ -401,7 +437,7 @@ Instead, these commands are available:
   (kill-all-local-variables)
   (buffer-disable-undo)
   (setq major-mode 'riece-user-list-mode
-        mode-name "User list"
+        mode-name "Users"
 	mode-line-buffer-identification
 	(riece-mode-line-buffer-identification
 	 '("Riece: " riece-long-channel-indicator " "))
